@@ -131,6 +131,15 @@ Global Variables and Constants
 
    A string indicating the name of the SGE implementation.
 
+.. data:: sge.SCALE_METHODS
+
+   A list of specific scale methods supported by the SGE implementation.
+
+   .. note::
+
+      This list does not include the generic scale methods, ``"noblur"``
+      and ``"smooth"``.  It is also possible for this list to be empty.
+
 .. data:: sge.BLEND_NORMAL
 
    Flag indicating normal blending.
@@ -305,6 +314,13 @@ cases.  No other drawing or projecting methods support anti-aliasing.
 Known Problems
 --------------
 
+Since the Pygame SGE uses Pygame, it necessarily inherits Pygame's bugs.
+Below, some notable bugs in Pygame 1.9.1 (the latest Pygame release) are
+indicated.
+
+Keyboard Lock-up
+~~~~~~~~~~~~~~~~
+
 There is a bug in either Pygame or SDL, most likely SDL, which sometimes
 causes keyboard input to stop working.  In Pygame programs such as this
 one, this occurs when pygame.display.set_mode is called multiple times,
@@ -332,6 +348,15 @@ you to provide some method of either exiting the game or exiting
 fullscreen with the mouse.  This can be a button somewhere on the screen
 if the game uses the mouse cursor, or it can be a simple mouse button
 click otherwise.
+
+Saving PNG Images
+~~~~~~~~~~~~~~~~~
+
+Some versions of Pygame have problems saving color information of PNG
+images correctly.  As a result, depending on your exact version of
+Pygame, images saves with :meth:`sge.gfx.Sprite.save` may be discolored.
+The best workaround is to choose a different format, such as BMP, GIF,
+or JPEG.
 """
 
 from __future__ import division
@@ -339,7 +364,7 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = "0.23"
+__version__ = "1.0"
 
 import sys
 import os
@@ -348,6 +373,7 @@ import pygame
 
 # Constants
 IMPLEMENTATION = "Pygame SGE"
+SCALE_METHODS = ["scale2x"]
 
 BLEND_NORMAL = None
 BLEND_ALPHA = 1
@@ -438,7 +464,8 @@ MOUSE_BUTTON_NAMES = {}
 for pair in MOUSE_BUTTONS.items():
     MOUSE_BUTTON_NAMES[pair[1]] = pair[0]
 
-from sge import collision, dsp, gfx, input, joystick, keyboard, mouse, snd, r
+from sge import (collision, dsp, gfx, input, joystick, keyboard, mouse, snd, s,
+                 r)
 
 
 __all__ = [
