@@ -117,11 +117,19 @@ def _scale(surface, width, height):
 
     width = int(round(width))
     height = int(round(height))
-    if sge.game.scale_smooth:
+    if sge.game.scale_method == "smooth":
         try:
             new_surf = pygame.transform.smoothscale(surface, (width, height))
         except (pygame.error, ValueError):
             new_surf = pygame.transform.scale(surface, (width, height))
+    elif sge.game.scale_method == "scale2x":
+        new_surf = surface
+        while (width / new_surf.get_width() >= 2 and
+               height / new_surf.get_height() >= 2):
+            new_surf = pygame.transform.scale2x(new_surf)
+
+        if new_surf.get_width() != width or new_surf.get_height() != height:
+            new_surf = pygame.transform.scale(surface, (width, height))
     else:
         new_surf = pygame.transform.scale(surface, (width, height))
 
@@ -185,7 +193,7 @@ def _set_mode():
     game_display_surface = _scale(game_display_surface, game.width, game.height)
     info = pygame.display.Info()
 
-    if sge.game.scale:
+    if game.scale:
         game_xscale = game.scale
         game_yscale = game.scale
 
@@ -193,18 +201,34 @@ def _set_mode():
         flags = pygame.FULLSCREEN | pygame.HWSURFACE | pygame.DOUBLEBUF
 
         modes = pygame.display.list_modes()
-        if modes != -1 and modes:
-            game_window_width, game_window_height = modes[0]
-        else:
+        if modes == -1 or not modes:
             w = "Couldn't find out the maximum resolution! Assuming 1024x768."
             warnings.warn(w)
-            game_window_width = 1024
-            game_window_height = 768
+            modes = [(1024, 768)]
+
+        force_auto_scale = False
+
+        if game.scale:
+            target_width = game.width * game.scale
+            target_height = game.height * game.scale
+            if (target_width, target_height) in modes:
+                game_window_width = target_width
+                game_window_height = target_height
+            else:
+                for mode in reversed(modes):
+                    if mode[0] >= target_width and mode[1] >= target_height:
+                        game_window_width, game_window_height = mode
+                        break
+                else:
+                    game_window_width, game_window_height = modes[0]
+                    force_auto_scale = True
+        else:
+            game_window_width, game_window_height = modes[0]
 
         game_window = pygame.display.set_mode(
             (game_window_width, game_window_height), flags)
 
-        if not game.scale:
+        if not game.scale or force_auto_scale:
             game_xscale = game_window_width / game.width
             game_yscale = game_window_height / game.height
 
@@ -477,7 +501,7 @@ def f_split_text(self, text, width=None):
 def o_update(self, time_passed, delta_mult):
     # Update this object (should be called each frame).
     # Update the animation frame.
-    if self.image_fps:
+    if self.image_fps and isinstance(self.sprite, sge.gfx.Sprite):
         self.rd["anim_count"] += time_passed
         self.image_index += int(self.rd["anim_count"] / self.rd["frame_time"])
         self.rd["anim_count"] %= abs(self.rd["frame_time"])
@@ -660,42 +684,46 @@ def o_detect_collisions(self):
 
 def o_get_origin_offset(self):
     # Return the amount to offset the origin as (x, y).
-    new_origin_x = self.sprite.origin_x
-    new_origin_y = self.sprite.origin_y
+    if isinstance(self.sprite, sge.gfx.Sprite):
+        new_origin_x = self.sprite.origin_x
+        new_origin_y = self.sprite.origin_y
 
-    img = s_get_image(self.sprite, self.image_index, self.image_xscale,
-                      self.image_yscale, self.image_rotation)
-    nimg = s_get_image(self.sprite, self.image_index, self.image_xscale,
-                       self.image_yscale)
-    width = img.get_width() / abs(self.image_xscale)
-    height = img.get_height() / abs(self.image_yscale)
-    normal_width = nimg.get_width() / abs(self.image_xscale)
-    normal_height = nimg.get_height() / abs(self.image_yscale)
+        img = s_get_image(self.sprite, self.image_index, self.image_xscale,
+                          self.image_yscale, self.image_rotation)
+        nimg = s_get_image(self.sprite, self.image_index, self.image_xscale,
+                           self.image_yscale)
+        width = img.get_width() / abs(self.image_xscale)
+        height = img.get_height() / abs(self.image_yscale)
+        normal_width = nimg.get_width() / abs(self.image_xscale)
+        normal_height = nimg.get_height() / abs(self.image_yscale)
 
-    if self.image_rotation % 360:
-        center_x = normal_width / 2
-        center_y = normal_height / 2
-        c_origin_x = new_origin_x - center_x
-        c_origin_y = new_origin_y - center_y
-        start_angle = math.atan2(c_origin_y, c_origin_x)
-        radius = math.hypot(c_origin_x, c_origin_y)
-        new_angle = start_angle + math.radians(self.image_rotation)
-        new_c_origin_x = radius * math.cos(new_angle)
-        new_c_origin_y = radius * math.sin(new_angle)
-        new_origin_x = new_c_origin_x + center_x
-        new_origin_y = new_c_origin_y + center_y
+        if self.image_rotation % 360:
+            center_x = normal_width / 2
+            center_y = normal_height / 2
+            c_origin_x = new_origin_x - center_x
+            c_origin_y = new_origin_y - center_y
+            start_angle = math.atan2(c_origin_y, c_origin_x)
+            radius = math.hypot(c_origin_x, c_origin_y)
+            new_angle = start_angle + math.radians(self.image_rotation)
+            new_c_origin_x = radius * math.cos(new_angle)
+            new_c_origin_y = radius * math.sin(new_angle)
+            new_origin_x = new_c_origin_x + center_x
+            new_origin_y = new_c_origin_y + center_y
 
-    if self.image_xscale < 0:
-        new_origin_x = width - new_origin_x
+        if self.image_xscale < 0:
+            new_origin_x = width - new_origin_x
 
-    if self.image_yscale < 0:
-        new_origin_y = height - new_origin_y
+        if self.image_yscale < 0:
+            new_origin_y = height - new_origin_y
 
-    new_origin_x *= abs(self.image_xscale)
-    new_origin_y *= abs(self.image_yscale)
+        new_origin_x *= abs(self.image_xscale)
+        new_origin_y *= abs(self.image_yscale)
 
-    x_offset = new_origin_x - self.sprite.origin_x
-    y_offset = new_origin_y - self.sprite.origin_y
+        x_offset = new_origin_x - self.sprite.origin_x
+        y_offset = new_origin_y - self.sprite.origin_y
+    else:
+        x_offset = 0
+        y_offset = 0
 
     return (x_offset, y_offset)
 
@@ -792,8 +820,8 @@ def r_update_pixelate(self, complete):
     transition_sprite = self.rd["t_sprite"]
     w = transition_sprite.width
     h = transition_sprite.height
-    smooth = sge.game.scale_smooth
-    sge.game.scale_smooth = False
+    scale_method = sge.game.scale_method
+    sge.game.scale_method = None
 
     if complete < 0.8:
         complete *= 1.25
@@ -811,7 +839,7 @@ def r_update_pixelate(self, complete):
         transition_sprite.draw_sprite(eraser, 0, 0, 0,
                                       blend_mode=sge.BLEND_RGBA_SUBTRACT)
 
-    sge.game.scale_smooth = smooth
+    sge.game.scale_method = scale_method
 
 
 def r_update_wipe_left(self, complete):
@@ -982,13 +1010,23 @@ def s_set_size(self):
     width = int(round(self.width))
     height = int(round(self.height))
     for i in six.moves.range(self.frames):
-        if sge.game.scale_smooth:
+        if sge.game.scale_method == "smooth":
             try:
                 self.rd["baseimages"][i] = pygame.transform.smoothscale(
                     self.rd["baseimages"][i], (width, height))
             except (pygame.error, ValueError):
                 self.rd["baseimages"][i] = pygame.transform.scale(
                     self.rd["baseimages"][i], (width, height))
+        elif sge.game.scale_method == "scale2x":
+            new_surf = surface
+            while (width / new_surf.get_width() >= 2 and
+                   height / new_surf.get_height() >= 2):
+                new_surf = pygame.transform.scale2x(new_surf)
+
+            if new_surf.get_width() != width or new_surf.get_height() != height:
+                new_surf = pygame.transform.scale(surface, (width, height))
+
+            self.rd["baseimages"][i] = new_surf
         else:
             self.rd["baseimages"][i] = pygame.transform.scale(
                 self.rd["baseimages"][i], (width, height))
@@ -1004,60 +1042,72 @@ def s_refresh(self):
 def s_set_transparency(self, image):
     # Return a copy of the surface with transparency properly set
     # for this sprite's settings.
-    if self.transparent and image.get_width() > 0:
+    if isinstance(self.transparent, sge.gfx.Color):
+        img = image.convert()
+        color = pygame.Color(*self.transparent)
+        img.set_colorkey(color, pygame.RLEACCEL)
+        return img
+    elif self.transparent and image.get_width() > 0:
         if image.get_flags() & pygame.SRCALPHA:
             return image.convert_alpha()
         else:
-            colorkey_img = image.convert()
+            img = image.convert()
             color = image.get_at((image.get_width() - 1, 0))
-            colorkey_img.set_colorkey(color, pygame.RLEACCEL)
-            return colorkey_img
+            img.set_colorkey(color, pygame.RLEACCEL)
+            return img
     else:
         return image.convert()
 
+    # Should not happen!
+    warnings.warn("Failed to account for transparency")
+    return image.convert()
+
 
 def s_get_image(self, num, xscale=1, yscale=1, rotation=0, alpha=255,
                 blend=None):
-    num %= self.frames
+    if isinstance(self, sge.gfx.Sprite):
+        num %= self.frames
 
-    i = ("s_image", self, self.rd["drawcycle"], num, xscale, yscale, rotation,
-         alpha, tuple(blend) if blend is not None else None)
-    img = cache.get(i)
-    if img is None:
-        if xscale != 0 and yscale != 0:
-            img = s_set_transparency(self, self.rd["baseimages"][num])
-            xflip = xscale < 0
-            yflip = yscale < 0
-            img = pygame.transform.flip(img, xflip, yflip)
-            img = _scale(img, self.width * abs(xscale),
-                         self.height * abs(yscale))
+        i = ("s_image", self, self.rd["drawcycle"], num, xscale, yscale,
+             rotation, alpha, tuple(blend) if blend is not None else None)
+        img = cache.get(i)
+        if img is None:
+            if xscale != 0 and yscale != 0:
+                img = s_set_transparency(self, self.rd["baseimages"][num])
+                xflip = xscale < 0
+                yflip = yscale < 0
+                img = pygame.transform.flip(img, xflip, yflip)
+                img = _scale(img, self.width * abs(xscale),
+                             self.height * abs(yscale))
 
-            if rotation != 0:
-                img = pygame.transform.rotate(img, -rotation)
+                if rotation != 0:
+                    img = pygame.transform.rotate(img, -rotation)
 
-            if alpha < 255:
-                if img.get_flags() & pygame.SRCALPHA:
-                    # Have to do this the more difficult way.
-                    img.fill((0, 0, 0, 255 - alpha), None,
-                             pygame.BLEND_RGBA_SUB)
-                else:
-                    img.set_alpha(alpha, pygame.RLEACCEL)
+                if alpha < 255:
+                    if img.get_flags() & pygame.SRCALPHA:
+                        # Have to do this the more difficult way.
+                        img.fill((0, 0, 0, 255 - alpha), None,
+                                 pygame.BLEND_RGBA_SUB)
+                    else:
+                        img.set_alpha(alpha, pygame.RLEACCEL)
 
-            if blend is not None:
-                img.fill(pygame.Color(*blend), None,
-                         pygame.BLEND_RGB_MULT)
-        else:
-            img = pygame.Surface((1, 1))
-            img.set_colorkey((0, 0, 0), pygame.RLEACCEL)
+                if blend is not None:
+                    img.fill(pygame.Color(*blend), None,
+                             pygame.BLEND_RGB_MULT)
+            else:
+                img = pygame.Surface((1, 1))
+                img.set_colorkey((0, 0, 0), pygame.RLEACCEL)
 
-    cache.add(i, img)
-    return img
+        cache.add(i, img)
+        return img
+    else:
+        return self
 
 
 def s_get_precise_mask(self, num, xscale, yscale, rotation):
     # Return a precise mask (2D list of True/False values) for the
     # given image index.
-    i = ("s_mask", hash(self), self.width, self.height, self.rd["drawcycle"],
+    i = ("s_mask", self, self.width, self.height, self.rd["drawcycle"],
          num, xscale, yscale, rotation)
     mask = cache.get(i)
     if mask is None:
@@ -1090,6 +1140,57 @@ def s_get_precise_mask(self, num, xscale, yscale, rotation):
     return mask
 
 
+def tg_blit(self, dest, x, y):
+    # Blit the tile grid onto a Pygame surface.
+    # Note: origin is NOT taken into account here!
+    def get_tile(i, j):
+        return self.tiles[i + j * self.section_length]
+
+    sx = x
+    sy = y
+
+    if self.render_method == "isometric":
+        x = -int(sx / self.tile_width)
+        imin = max(0, x)
+        imax = min(self.section_length,
+                   x + int(math.ceil(dest.get_width() / self.tile_width)) + 1)
+
+        h = self.tile_height / 2
+        y = -int(sy / h)
+        jmin = max(0, y)
+        jmax = min(int(len(self.tiles) // self.section_length),
+                   y + int(math.ceil(dest.get_height() / h)) + 1)
+    else:
+        x = -int(sx / self.tile_width)
+        imin = max(0, x)
+        imax = min(self.section_length,
+                   x + int(math.ceil(dest.get_width() / self.tile_width)) + 1)
+
+        y = -int(sy / self.tile_height)
+        jmin = max(0, y)
+        jmax = min(int(len(self.tiles) // self.section_length),
+                   y + int(math.ceil(dest.get_height() / self.tile_height)) + 1)
+
+    irng = six.moves.range(imin, imax)
+    jrng = six.moves.range(jmin, jmax)
+
+    for i in irng:
+        for j in jrng:
+            sprite = get_tile(i, j)
+            if sprite is not None:
+                if self.render_method == "isometric":
+                    x = sx + i * self.tile_width
+                    y = sx + j * self.tile_height / 2
+                    if i / 2 != i // 2:
+                        x += i * self.tile_width / 2
+                else:
+                    x = sx + i * self.tile_width
+                    y = sy + j * self.tile_height
+
+                ssurf = s_set_transparency(sprite, sprite.rd["baseimages"][0])
+                dest.blit(ssurf, (x, y))
+
+
 def v_limit(self):
     # Keep the view within the room.
     if sge.game.current_room is not None:
